@@ -11,9 +11,12 @@
 #include <iostream>
 #include <sstream>
 #include <map>
+#include <unistd.h>
 
 #include "cmd.hh"
 #include "app_state.hh"
+#include "automate_ostream.hh"
+#include "automate_reader.hh"
 #include "ui.hh"
 #include "lua.hh"
 #include "lua_hooks.hh"
@@ -36,16 +39,19 @@ CMD_GROUP(automate, "automate", "au", CMD_REF(automation),
 
 namespace commands {
   automate::automate(string const & name,
+                     bool stdio_ok,
+                     bool hidden,
                      string const & params,
                      string const & abstract,
                      string const & desc,
                      options::options_type const & opts) :
-    command(name, "", CMD_REF(automate), false, false, params, abstract,
+    command(name, "", CMD_REF(automate), false, hidden, params, abstract,
             // We set use_workspace_options true, because all automate
             // commands need a database, and they expect to get the database
             // name from the workspace options, even if they don't need a
             // workspace for anything else.
-            desc, true, opts, false)
+            desc, true, opts, false),
+    stdio_ok(stdio_ok)
   {
   }
 
@@ -66,12 +72,22 @@ namespace commands {
   {
     exec(app, execid, args, std::cout);
   }
+
+  bool
+  automate::can_run_from_stdio() const
+  {
+    return stdio_ok;
+  }
 }
 
 // This number is only raised once, during the process of releasing a new
 // version of monotone, by the release manager. For more details, see
 // point (2) in notes/release-checklist.txt
-static string const interface_version = "11.0";
+static string const interface_version = "12.0";
+
+// This number determines the format version of the stdio packet format.
+// The original format which came without a version notification was "1".
+static string const stdio_format_version = "2";
 
 // Name: interface_version
 // Arguments: none
@@ -94,6 +110,60 @@ CMD_AUTOMATE(interface_version, "",
   output << interface_version << '\n';
 }
 
+// these headers are outputted before any other output for stdio and remote_stdio
+void commands::get_stdio_headers(std::vector<std::pair<std::string,std::string> > & headers)
+{
+    headers.push_back(make_pair("format-version", stdio_format_version));
+}
+
+// Name: bandtest
+// Arguments: { info | warning | error | fatal | ticker }
+// Added in: FIXME
+// Purpose: Emulates certain kinds of diagnostic / UI messages for debugging
+//          and testing purposes
+// Output format: None
+// Error conditions: None.
+CMD_AUTOMATE_HIDDEN(bandtest, "{ info | warning | error | ticker }",
+             N_("Emulates certain kinds of diagnostic / UI messages "
+                "for debugging and testing purposes, such as stdio"),
+             "",
+             options::opts::none)
+{
+  E(args.size() == 1, origin::user,
+    F("wrong argument count"));
+
+  std::string type = args.at(0)();
+  if (type.compare("info") == 0)
+    P(F("this is an informational message"));
+  else if (type.compare("warning") == 0)
+    W(F("this is a warning"));
+  else if (type.compare("error") == 0)
+    E(false, origin::user, F("this is an error message"));
+  else if (type.compare("ticker") == 0)
+    {
+      ticker first("fake ticker (not fixed)", "f1", 3);
+      ticker second("fake ticker (fixed)", "f2", 5);
+
+      int max = 20;
+      second.set_total(max);
+
+      for (int i=0; i<max; i++)
+        {
+          first+=3;
+          ++second;
+          usleep(100000); // 100ms
+        }
+    }
+  else
+    I(false);
+}
+
+
+static void out_of_band_to_automate_streambuf(char channel, std::string const& text, void *opaque)
+{
+  reinterpret_cast<automate_ostream*>(opaque)->write_out_of_band(channel, text);
+}
+
 // Name: stdio
 // Arguments: none
 // Added in: 1.0
@@ -109,14 +179,16 @@ CMD_AUTOMATE(interface_version, "",
 //     l6:leavese
 //     l7:parents40:0e3171212f34839c2e3263e7282cdeea22fc5378e
 //
-// Output format: <command number>:<err code>:<last?>:<size>:<output>
+// Output format: <command number>:<err code>:<stream>:<size>:<output>
 //   <command number> is a decimal number specifying which command
 //   this output is from. It is 0 for the first command, and increases
 //   by one each time.
 //   <err code> is 0 for success, 1 for a syntax error, and 2 for any
 //   other error.
-//   <last?> is 'l' if this is the last piece of output for this command,
-//   and 'm' if there is more output to come.
+//   <stream> is 'l' if this is the last piece of output for this command,
+//   and 'm' if there is more output to come. Otherwise, 'e', 'p' and 'w'
+//   notify the caller about errors, informational messages and warnings.
+//   A special type 't' outputs progress information for long-term actions.
 //   <size> is the number of bytes in the output.
 //   <output> is the output of the command.
 //   Example:
@@ -130,227 +202,10 @@ CMD_AUTOMATE(interface_version, "",
 // Error conditions: Errors encountered by the commands run only set
 //   the error code in the output for that command. Malformed input
 //   results in exit with a non-zero return value and an error message.
-
-// automate_streambuf and automate_ostream are so we can dump output at a
-// set length, rather than waiting until we have all of the output.
-
-
-class automate_reader
-{
-  istream & in;
-  enum location {opt, cmd, none, eof};
-  location loc;
-  bool get_string(std::string & out)
-  {
-    out.clear();
-    if (loc == none || loc == eof)
-      {
-        return false;
-      }
-    size_t size(0);
-    char c;
-    read(&c, 1);
-    if (c == 'e')
-      {
-        loc = none;
-        return false;
-      }
-    while(c <= '9' && c >= '0')
-      {
-        size = (size*10)+(c-'0');
-        read(&c, 1);
-      }
-    E(c == ':', origin::user,
-        F("Bad input to automate stdio: expected ':' after string size"));
-    char *str = new char[size];
-    size_t got = 0;
-    while(got < size)
-      {
-        int n = read(str+got, size-got);
-        got += n;
-      }
-    out = std::string(str, size);
-    delete[] str;
-    L(FL("Got string '%s'") % out);
-    return true;
-  }
-  std::streamsize read(char *buf, size_t nbytes, bool eof_ok = false)
-  {
-    std::streamsize rv;
-
-    rv = in.rdbuf()->sgetn(buf, nbytes);
-
-    E(eof_ok || rv > 0, origin::user,
-      F("Bad input to automate stdio: unexpected EOF"));
-    return rv;
-  }
-  void go_to_next_item()
-  {
-    if (loc == eof)
-      return;
-    string starters("ol");
-    string whitespace(" \r\n\t");
-    string foo;
-    while (loc != none)
-      get_string(foo);
-    char c('e');
-    do
-      {
-        if (read(&c, 1, true) == 0)
-          {
-            loc = eof;
-            return;
-          }
-      }
-    while (whitespace.find(c) != std::string::npos);
-    switch (c)
-      {
-      case 'o': loc = opt; break;
-      case 'l': loc = cmd; break;
-      default:
-        E(false, origin::user,
-            F("Bad input to automate stdio: unknown start token '%c'") % c);
-      }
-  }
-public:
-  automate_reader(istream & is) : in(is), loc(none)
-  {}
-  bool get_command(vector<pair<string, string> > & params,
-                   vector<string> & cmdline)
-  {
-    params.clear();
-    cmdline.clear();
-    if (loc == none)
-      go_to_next_item();
-    if (loc == eof)
-      return false;
-    else if (loc == opt)
-      {
-        string key, val;
-        while(get_string(key) && get_string(val))
-          params.push_back(make_pair(key, val));
-        go_to_next_item();
-      }
-    E(loc == cmd, origin::user,
-      F("Bad input to automate stdio: expected '%c' token") % cmd);
-    string item;
-    while (get_string(item))
-      {
-        cmdline.push_back(item);
-      }
-    E(cmdline.size() > 0, origin::user,
-        F("Bad input to automate stdio: command name is missing"));
-    return true;
-  }
-  void reset()
-  {
-    loc = none;
-  }
-};
-
-struct automate_streambuf : public std::streambuf
-{
-private:
-  size_t _bufsize;
-  std::ostream *out;
-  automate_reader *in;
-  int cmdnum;
-  int err;
-public:
-  automate_streambuf(size_t bufsize)
-    : std::streambuf(), _bufsize(bufsize), out(0), in(0), cmdnum(0), err(0)
-  {
-    char *inbuf = new char[_bufsize];
-    setp(inbuf, inbuf + _bufsize);
-  }
-  automate_streambuf(std::ostream & o, size_t bufsize)
-    : std::streambuf(), _bufsize(bufsize), out(&o), in(0), cmdnum(0), err(0)
-  {
-    char *inbuf = new char[_bufsize];
-    setp(inbuf, inbuf + _bufsize);
-  }
-  automate_streambuf(automate_reader & i, size_t bufsize)
-    : std::streambuf(), _bufsize(bufsize), out(0), in(&i), cmdnum(0), err(0)
-  {
-    char *inbuf = new char[_bufsize];
-    setp(inbuf, inbuf + _bufsize);
-  }
-  ~automate_streambuf()
-  {}
-
-  void set_err(int e)
-  {
-    sync();
-    err = e;
-  }
-
-  void end_cmd()
-  {
-    _M_sync(true);
-    ++cmdnum;
-    err = 0;
-  }
-
-  virtual int sync()
-  {
-    _M_sync();
-    return 0;
-  }
-
-  void _M_sync(bool end = false)
-  {
-    if (!out)
-      {
-        setp(pbase(), pbase() + _bufsize);
-        return;
-      }
-    int num = pptr() - pbase();
-    if (num || end)
-      {
-        (*out) << cmdnum << ':'
-            << err << ':'
-            << (end?'l':'m') << ':'
-            << num << ':' << std::string(pbase(), num);
-        setp(pbase(), pbase() + _bufsize);
-        out->flush();
-      }
-  }
-  int_type
-  overflow(int_type c = traits_type::eof())
-  {
-    sync();
-    sputc(c);
-    return 0;
-  }
-};
-
-struct automate_ostream : public std::ostream
-{
-  automate_streambuf _M_autobuf;
-
-  automate_ostream(std::ostream &out, size_t blocksize)
-    : std::ostream(NULL),
-      _M_autobuf(out, blocksize)
-  { this->init(&_M_autobuf); }
-
-  ~automate_ostream()
-  {}
-
-  automate_streambuf *
-  rdbuf() const
-  { return const_cast<automate_streambuf *>(&_M_autobuf); }
-
-  void set_err(int e)
-  { _M_autobuf.set_err(e); }
-
-  void end_cmd()
-  { _M_autobuf.end_cmd(); }
-};
-
-CMD_AUTOMATE(stdio, "",
-             N_("Automates several commands in one run"),
-             "",
-             options::opts::automate_stdio_size)
+CMD_AUTOMATE_NO_STDIO(stdio, "",
+                      N_("Automates several commands in one run"),
+                      "",
+                      options::opts::automate_stdio_size)
 {
   E(args.empty(), origin::user,
     F("no arguments needed"));
@@ -361,20 +216,32 @@ CMD_AUTOMATE(stdio, "",
   // immediately if a version discrepancy exists
   db.ensure_open();
 
+  // disable user prompts, f.e. for password decryption
+  app.opts.non_interactive = true;
   options original_opts = app.opts;
 
   automate_ostream os(output, app.opts.automate_stdio_size);
   automate_reader ar(std::cin);
+
+  std::vector<std::pair<std::string, std::string> > headers;
+  commands::get_stdio_headers(headers);
+  os.write_headers(headers);
+
   vector<pair<string, string> > params;
   vector<string> cmdline;
+  global_sanity.set_out_of_band_handler(&out_of_band_to_automate_streambuf, &os);
+
   while (true)
     {
-      command const * cmd = 0;
+      automate const * acmd = 0;
       command_id id;
       args_vector args;
 
+      // FIXME: what follows is largely duplicated
+      // in network/automate_session.cc::do_work()
+      //
       // stdio decoding errors should be noted with errno 1,
-      // errno 2 is reserved for errors coming from the commands itself
+      // errno 2 is reserved for errors from the commands itself
       try
         {
           if (!ar.get_command(params, cmdline))
@@ -407,9 +274,15 @@ CMD_AUTOMATE(stdio, "",
           for (command_id::size_type i = 0; i < id.size(); i++)
             args.erase(args.begin());
 
-          cmd = CMD_REF(automate)->find_command(id);
+          command const * cmd = CMD_REF(automate)->find_command(id);
           I(cmd != NULL);
 
+          acmd = dynamic_cast< automate const * >(cmd);
+          I(acmd != NULL);
+
+          E(acmd->can_run_from_stdio(), origin::network,
+            F("sorry, that can't be run remotely or over stdio"));
+
           if (cmd->use_workspace_options())
             {
               // Re-read the ws options file, rather than just copying
@@ -422,42 +295,43 @@ CMD_AUTOMATE(stdio, "",
           options::options_type opts;
           opts = options::opts::globals() | cmd->opts();
           opts.instantiate(&app.opts).from_key_value_pairs(params);
+
+          // set a fixed ticker type regardless what the user wants to
+          // see, because anything else would screw the stdio-encoded output
+          ui.set_tick_write_stdio();
         }
       // FIXME: we need to re-package and rethrow this special exception
       // since it is not based on informative_failure
       catch (option::option_error & e)
         {
-          os.set_err(1);
-          os<<e.what();
-          os.end_cmd();
+          os.write_out_of_band('e', e.what());
+          os.end_cmd(1);
           ar.reset();
           continue;
         }
       catch (recoverable_failure & f)
         {
-          os.set_err(1);
-          os<<f.what();
-          os.end_cmd();
+          os.write_out_of_band('e', f.what());
+          os.end_cmd(1);
           ar.reset();
           continue;
         }
 
       try
         {
-          automate const * acmd = dynamic_cast< automate const * >(cmd);
-          I(acmd);
           acmd->exec_from_automate(app, id, args, os);
-          // set app.opts to the originally given options
-          // so the next command has an identical setup
+          os.end_cmd(0);
+
+          // restore app.opts
           app.opts = original_opts;
         }
-      catch(recoverable_failure & f)
+      catch (recoverable_failure & f)
         {
-          os.set_err(2);
-          os<<f.what();
+          os.write_out_of_band('e', f.what());
+          os.end_cmd(2);
         }
-      os.end_cmd();
     }
+    global_sanity.set_out_of_band_handler();
 }
 
 LUAEXT(mtn_automate, )
@@ -493,6 +367,9 @@ LUAEXT(mtn_automate, )
           args.push_back(next_arg);
         }
 
+      // disable user prompts, f.e. for password decryption
+      app_p->opts.non_interactive = true;
+
       options::options_type opts;
       opts = options::opts::all_options() - options::opts::globals();
       opts.instantiate(&app_p->opts).reset();
@@ -576,7 +453,7 @@ LUAEXT(mtn_automate, )
   os.flush();
 
   lua_pushboolean(LS, result);
-  lua_pushstring(LS, output.str().c_str());
+  lua_pushlstring(LS, output.str().data(), output.str().size());
   return 2;
 }
 
