@@ -9,7 +9,6 @@
 // PURPOSE.
 
 #include "base.hh"
-#include <deque>
 #include <map>
 #include <iostream>
 #include <sstream>
@@ -32,7 +31,6 @@
 #include "roster.hh"
 
 using std::cout;
-using std::deque;
 using std::make_pair;
 using std::map;
 using std::ostream;
@@ -726,57 +724,6 @@ struct rev_cmp
   }
 };
 
-struct revision_loader
-{
-  database & db;
-  log_direction direction;
-
-  revision_loader(database & db, log_direction const direction) :
-    db(db), direction(direction) {}
-
-  void
-  load_related_revs(revision_id const & rid, set<revision_id> & relatives)
-  {
-    switch (direction)
-      {
-      case log_forward: // optional with --next N
-        db.get_revision_children(rid, relatives);
-        break;
-      case log_reverse: // default and with --last N
-        db.get_revision_parents(rid, relatives);
-        break;
-      }
-  }
-
-  void
-  load_implied_revs(set<revision_id> & revs)
-  {
-    std::deque<revision_id> next(revs.begin(), revs.end());
-
-    while (!next.empty())
-      {
-        revision_id const & rid(next.front());
-        MM(rid);
-
-        set<revision_id> relatives;
-        MM(relatives);
-        load_related_revs(rid, relatives);
-
-        for (set<revision_id>::const_iterator i = relatives.begin();
-             i != relatives.end(); ++i)
-          {
-            if (null_id(*i))
-              continue;
-            pair<set<revision_id>::iterator, bool> res = revs.insert(*i);
-            if (res.second)
-              next.push_back(*i);
-          }
-
-        next.pop_front();
-      }
-  }
-};
-
 typedef priority_queue<pair<rev_height, revision_id>,
                        vector<pair<rev_height, revision_id> >,
                        rev_cmp> frontier_t;
@@ -788,7 +735,6 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
     options::opts::last | options::opts::next |
     options::opts::from | options::opts::to | options::opts::revision |
     options::opts::brief | options::opts::diffs |
-    options::opts::format_dates | options::opts::date_fmt |
     options::opts::depth | options::opts::exclude |
     options::opts::no_merges | options::opts::no_files |
     options::opts::no_graph)
@@ -802,7 +748,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       if (!app.opts.date_fmt.empty())
         date_fmt = app.opts.date_fmt;
       else
-        app.lua.hook_get_date_format_spec(date_fmt);
+        app.lua.hook_get_date_format_spec(date_time_short, date_fmt);
     }
 
   long last = app.opts.last;
@@ -816,7 +762,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
   if (next >= 0)
     direction = log_forward;
 
-  revision_loader loader(db, direction);
+  graph_loader loader(db);
 
   rev_cmp cmp(direction);
   frontier_t frontier(cmp);
@@ -872,7 +818,16 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           ending_revs.insert(rids.begin(), rids.end());
         }
 
-      loader.load_implied_revs(ending_revs);
+      if (direction == log_forward)
+        {
+          loader.load_descendants(ending_revs);
+        }
+      else if (direction == log_reverse)
+        {
+          loader.load_ancestors(ending_revs);
+        }
+      else
+        I(false);
     }
 
   L(FL("%d ending revisions") % ending_revs.size());
@@ -1008,12 +963,12 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
 
               if (mask.includes(roster, node))
                 {
-                  marked_revs.insert(marks.file_content.begin(),
-                                     marks.file_content.end());
-                  marked_revs.insert(marks.parent_name.begin(),
-                                     marks.parent_name.end());
+                  marked_revs.insert(marks->file_content.begin(),
+                                     marks->file_content.end());
+                  marked_revs.insert(marks->parent_name.begin(),
+                                     marks->parent_name.end());
                   for (map<attr_key, set<revision_id> >::const_iterator
-                         a = marks.attrs.begin(); a != marks.attrs.end(); ++a)
+                         a = marks->attrs.begin(); a != marks->attrs.end(); ++a)
                     marked_revs.insert(a->second.begin(), a->second.end());
                 }
             }
@@ -1059,10 +1014,16 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
         {
           interesting.insert(marked_revs.begin(), marked_revs.end());
         }
-      else
+      else if (direction == log_forward)
         {
-          loader.load_related_revs(rid, interesting);
+          loader.load_children(rid, interesting);
         }
+      else if (direction == log_reverse)
+        {
+          loader.load_parents(rid, interesting);
+        }
+      else
+        I(false);
 
       if (print_this)
         {
@@ -1087,7 +1048,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
           else
             {
               out << string(65, '-') << '\n';
-              out << "Revision: " << rid << '\n';
+              out << _("Revision: ") << rid << '\n';
 
               changes_summary csum;
 
@@ -1102,12 +1063,12 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
 
               for (set<revision_id>::const_iterator anc = ancestors.begin();
                    anc != ancestors.end(); ++anc)
-                out << "Ancestor: " << *anc << '\n';
+                out << _("Ancestor: ") << *anc << '\n';
 
-              log_certs(certs, out, author_name, "Author: ", false);
-              log_date_certs(certs, out, date_fmt, "Date: ", false);
-              log_certs(certs, out, branch_name, "Branch: ", false);
-              log_certs(certs, out, tag_name,    "Tag: ",    false);
+              log_certs(certs, out, author_name,   _("Author: "), false);
+              log_date_certs(certs, out, date_fmt, _("Date: "), false);
+              log_certs(certs, out, branch_name,   _("Branch: "), false);
+              log_certs(certs, out, tag_name,      _("Tag: "),    false);
 
               if (!app.opts.no_files && !csum.cs.empty())
                 {
@@ -1116,8 +1077,8 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
                   out << '\n';
                 }
 
-              log_certs(certs, out, changelog_name, "ChangeLog: ", true);
-              log_certs(certs, out, comment_name,   "Comments: ",  true);
+              log_certs(certs, out, changelog_name, _("ChangeLog: "), true);
+              log_certs(certs, out, comment_name,   _("Comments: "),  true);
             }
 
           if (app.opts.diffs)
