@@ -75,14 +75,22 @@ struct sanity::impl
   bool already_dumping;
   std::vector<MusingI const *> musings;
 
+  void (*out_of_band_function)(char channel, std::string const& text, void *opaque);
+  void *out_of_band_opaque;
+
   impl() :
     debug(false), quiet(false), reallyquiet(false), logbuf(0xffff),
-    already_dumping(false)
+    already_dumping(false), out_of_band_function(0), out_of_band_opaque(0)
   {}
 };
 
 // debugging / logging system
 
+sanity::sanity()
+{
+  imp = 0;
+}
+
 sanity::~sanity()
 {
   if (imp)
@@ -279,6 +287,9 @@ sanity::progress(i18n_format const & i18nfmt,
 {
   string str = do_format(i18nfmt, file, line);
 
+  if (maybe_write_to_out_of_band_handler('p', str))
+    return;
+
   if (str.size() > constants::log_line_sz)
     {
       str.resize(constants::log_line_sz);
@@ -298,6 +309,9 @@ sanity::warning(i18n_format const & i18nfmt,
 {
   string str = do_format(i18nfmt, file, line);
 
+  if (maybe_write_to_out_of_band_handler('w', str))
+    return;
+
   if (str.size() > constants::log_line_sz)
     {
       str.resize(constants::log_line_sz);
@@ -439,6 +453,22 @@ sanity::gasp()
   imp->already_dumping = false;
 }
 
+void sanity::set_out_of_band_handler(void (*out_of_band_function)(char, std::string const&, void *), void *opaque_data)
+{
+  imp->out_of_band_function= out_of_band_function;
+  imp->out_of_band_opaque= opaque_data;
+}
+
+bool sanity::maybe_write_to_out_of_band_handler(char channel, std::string const& str)
+{
+  if (imp->out_of_band_function)
+    {
+      (*imp->out_of_band_function)(channel, str, imp->out_of_band_opaque);
+      return true;
+    }
+  return false;
+}
+
 template <> void
 dump(string const & obj, string & out)
 {
@@ -449,6 +479,11 @@ dump(char const * const & obj, string & out)
 {
   out = obj;
 }
+template<> void
+dump(bool const & obj, string & out)
+{
+  out = (obj ? "true" : "false");
+}
 
 void
 sanity::print_var(std::string const & value, char const * var,
