@@ -1,7 +1,7 @@
-
 mtn_setup()
 
-check(raw_mtn("--norc"), 2, false)
+check(raw_mtn("--norc"), 2, false, true)
+check(qgrep("Usage:", "stderr"))
 check(raw_mtn("--help"), 0, false)
 check(raw_mtn("--version"), 0, false)
 check(raw_mtn("--nostd", "--help"), 0, false)
