@@ -6,6 +6,7 @@ end
 
 addfile("testfile", C("abc"))
 addfile("otherfile", C("123"))
+addfile("unchanging", "asdf\n")
 commit()
 root_rev = base_revision()
 
@@ -19,6 +20,12 @@ writefile("otherfile", C("a2c"))
 commit()
 second_rev = base_revision()
 
+revert_to(root_rev)
+check(mtn("pluck", "-r", second_rev, "unchanging"), 1, false, true)
+check(qgrep("no changes", "stderr"))
+check(readfile("testfile") == C("abc"))
+check(readfile("otherfile") == C("123"))
+
 revert_to(root_rev)
 check(readfile("testfile") == C("abc"))
 check(readfile("otherfile") == C("123"))
